package com.marvinelsen.willow.ui.controllers

import com.marvinelsen.willow.Model
import com.marvinelsen.willow.config.Config
import com.marvinelsen.willow.config.Script
import com.marvinelsen.willow.ui.DictionaryEntryFx
import com.marvinelsen.willow.ui.cells.DictionaryEntryCellFactory
import javafx.beans.binding.Bindings
import javafx.fxml.FXML
import javafx.scene.control.Label
import javafx.scene.control.ListView
import javafx.scene.control.TabPane
import javafx.scene.web.WebView
import kotlinx.html.body
import kotlinx.html.h1
import kotlinx.html.html
import kotlinx.html.li
import kotlinx.html.ol
import kotlinx.html.stream.createHTML
import java.util.ResourceBundle

class DetailsController(private val model: Model, private val config: Config) {
    @FXML
    private lateinit var resources: ResourceBundle

    @FXML
    private lateinit var tabPaneDetails: TabPane

    @FXML
    private lateinit var webViewDefinition: WebView

    @FXML
    @Suppress("UnusedPrivateProperty")
    private lateinit var listviewSentences: ListView<DictionaryEntryFx>

    @FXML
    private lateinit var listViewWords: ListView<DictionaryEntryFx>

    @FXML
    @Suppress("UnusedPrivateProperty")
    private lateinit var listViewCharacters: ListView<DictionaryEntryFx>

    @FXML
    private lateinit var labelHeadword: Label

    @FXML
    @Suppress("UnusedPrivateMember")
    private fun initialize() {
        labelHeadword.textProperty().bind(
            Bindings.createStringBinding(
                {
                    when (config.script.value!!) {
                        Script.SIMPLIFIED -> model.selectedEntry.value?.simplifiedProperty?.value
                        Script.TRADITIONAL -> model.selectedEntry.value?.traditionalProperty?.value
                    }
                },
                config.script,
                model.selectedEntry
            )
        )
        labelHeadword
            .styleProperty()
            .bind(
                Bindings.concat(
                    "-fx-font-size: ",
                    config.detailHeadwordFontSize.asString(),
                    "px;"
                )
            )

        tabPaneDetails.disableProperty().bind(Bindings.isNull(model.selectedEntry))

        listViewWords.cellFactory = DictionaryEntryCellFactory(resources, config)
        listViewWords.items = model.wordsContaining
        tabPaneDetails.selectionModel.selectedItemProperty().addListener { _, _, selectedTab ->
            if (model.selectedEntry.value == null) return@addListener

            when (selectedTab.id) {
                "tabWords" -> {
                    model.findWords()
                }

                else -> {}
            }
        }

        webViewDefinition.apply {
            isContextMenuEnabled = false
            engine.userStyleSheetLocation =
                this::class.java.getResource("/css/definitions.css")!!.toExternalForm()
        }

        model.selectedEntry.addListener { _, _, newValue ->
            if (newValue == null) {
                return@addListener
            }
            when (tabPaneDetails.selectionModel.selectedItem.id) {
                "tabWords" -> {
                    model.findWords()
                }

                else -> {}
            }
            webViewDefinition.engine.loadContent(
                createHTML().html {
                    body {
                        h1 {
                            +"CC-CEDICT"
                        }
                        ol {
                            for (definition in newValue.definitions) {
                                li {
                                    +definition.joinToString(separator = "; ")
                                }
                            }
                        }
                    }
                }
            )
        }
    }
}
