package com.marvinelsen.willow.ui.dialogs

import com.marvinelsen.willow.WillowApplication
import com.marvinelsen.willow.config.Config
import com.marvinelsen.willow.config.Pronunciation
import com.marvinelsen.willow.config.Script
import com.marvinelsen.willow.config.Theme
import com.marvinelsen.willow.ui.formatters.FontSizeTextFormatter
import javafx.fxml.FXML
import javafx.fxml.FXMLLoader
import javafx.scene.control.ButtonType
import javafx.scene.control.CheckBox
import javafx.scene.control.ComboBox
import javafx.scene.control.Dialog
import javafx.scene.control.DialogPane
import javafx.scene.control.Spinner
import javafx.stage.Modality
import javafx.stage.Stage
import javafx.stage.Window
import javafx.util.Callback
import java.util.Locale

class PreferencesDialog(owner: Window?, config: Config) : Dialog<PreferencesDialog.Result>() {
    companion object {
        private const val DIALOG_MIN_HEIGHT = 400.0
        private const val DIALOG_MIN_WIDTH = 400.0
    }

    enum class Result {
        CHANGES, NO_CHANGES
    }

    @FXML
    private lateinit var comboBoxPronunciationDetails: ComboBox<Pronunciation>

    @FXML
    private lateinit var comboBoxLocale: ComboBox<Locale>

    @FXML
    private lateinit var comboBoxTheme: ComboBox<Theme>

    @FXML
    private lateinit var comboBoxScript: ComboBox<Script>

    @FXML
    private lateinit var comboBoxPronunciationSearchResults: ComboBox<Pronunciation>

    @FXML
    private lateinit var checkBoxShowPronunciationSearchResults: CheckBox

    @FXML
    private lateinit var checkBoxShowDefinitionSearchResults: CheckBox

    @FXML
    private lateinit var spinnerHeadwordFontSizeDetails: Spinner<Int>

    @FXML
    private lateinit var spinnerPronunciationFontSizeDetails: Spinner<Int>

    @FXML
    private lateinit var spinnerHeadwordFontSizeSearchResults: Spinner<Int>

    @FXML
    private lateinit var spinnerPronunciationFontSizeSearchResults: Spinner<Int>

    @FXML
    private lateinit var spinnerDefinitionFontSizeSearchResults: Spinner<Int>

    private val entryHeadwordFontSizeObjectProperty = config.detailHeadwordFontSize.asObject()
    private val entryPronunciationFontSizeObjectProperty = config.detailPronunciationFontSize.asObject()

    private val searchResultHeadwordFontSizeObjectProperty = config.searchResults.headwordFontSize.asObject()
    private val searchResultPronunciationFontSizeObjectProperty = config.searchResults.pronunciationFontSize.asObject()
    private val searchResultDefinitionFontSizeObjectProperty = config.searchResults.definitionFontSize.asObject()

    init {
        val loader = FXMLLoader(WillowApplication::class.java.getResource("/fxml/dialogs/preferences.fxml"))
        loader.setController(this)
        val root: DialogPane = loader.load()

        dialogPane = root
        // TODO: Use resource bundle string
        title = "Settings"
        isResizable = true

        initOwner(owner)
        initModality(Modality.APPLICATION_MODAL)

        (dialogPane.scene.window as Stage).apply {
            minWidth = DIALOG_MIN_WIDTH
            minHeight = DIALOG_MIN_HEIGHT
        }

        comboBoxLocale.items.addAll(
            setOf(
                Locale.ENGLISH,
                Locale.GERMAN,
                Locale.CHINA,
                Locale.TAIWAN
            )
        )

        comboBoxTheme.valueProperty().bindBidirectional(config.theme)
        comboBoxScript.valueProperty().bindBidirectional(config.script)
        comboBoxLocale.valueProperty().bindBidirectional(config.locale)
        comboBoxPronunciationSearchResults.valueProperty().bindBidirectional(config.searchResults.pronunciation)

        checkBoxShowDefinitionSearchResults
            .selectedProperty()
            .bindBidirectional(config.searchResults.shouldShowDefinition)
        checkBoxShowPronunciationSearchResults
            .selectedProperty()
            .bindBidirectional(config.searchResults.shouldShowPronunciation)

        with(spinnerHeadwordFontSizeDetails) {
            editor.textFormatter = FontSizeTextFormatter()
            valueFactory.valueProperty().bindBidirectional(entryHeadwordFontSizeObjectProperty)
        }

        with(spinnerPronunciationFontSizeDetails) {
            editor.textFormatter = FontSizeTextFormatter()
            valueFactory.valueProperty().bindBidirectional(entryPronunciationFontSizeObjectProperty)
        }

        with(spinnerHeadwordFontSizeSearchResults) {
            editor.textFormatter = FontSizeTextFormatter()
            valueFactory.valueProperty().bindBidirectional(searchResultHeadwordFontSizeObjectProperty)
        }

        with(spinnerPronunciationFontSizeSearchResults) {
            editor.textFormatter = FontSizeTextFormatter()
            valueFactory.valueProperty().bindBidirectional(searchResultPronunciationFontSizeObjectProperty)
        }

        with(spinnerDefinitionFontSizeSearchResults) {
            editor.textFormatter = FontSizeTextFormatter()
            valueFactory.valueProperty().bindBidirectional(searchResultDefinitionFontSizeObjectProperty)
        }

        resultConverter = Callback(::convertToResult)
    }

    private fun convertToResult(buttonType: ButtonType) =
        when (buttonType) {
            ButtonType.APPLY -> Result.CHANGES
            else -> Result.NO_CHANGES
        }
}
